package com.iorange.hdrcam;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import android.content.Context;
import android.hardware.Camera;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.ShutterCallback;
import android.hardware.Camera.Size;
import android.os.Environment;
import android.util.AttributeSet;
import android.view.SurfaceHolder;
import android.view.SurfaceView;


public class PreviewSurface extends SurfaceView implements SurfaceHolder.Callback
{
	private SurfaceHolder m_Holder;
	private Camera m_Camera = null;
	private Boolean m_CanDoPhoto = false;
	
	public PreviewSurface(Context context, AttributeSet attrs)
	{
		super(context, attrs);
		
		m_Holder = getHolder();
		m_Holder.addCallback(this);
		m_Holder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
	}
	
	/* Callback invoked when the surface dimensions change. */
	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height)
	{
		Camera.Parameters parameters = m_Camera.getParameters();

		List<Size> sizes = parameters.getSupportedPreviewSizes();
		Size optimalSize = getOptimalPreviewSize(sizes, width, height);
		parameters.setPreviewSize(optimalSize.width, optimalSize.height);

		m_Camera.setParameters(parameters);
		m_Camera.startPreview();
		m_CanDoPhoto = true;
	}

	/*
     * Callback invoked when the Surface has been created and is ready to be
     * used.
     */
	@Override
	public void surfaceCreated(SurfaceHolder holder)
	{
		m_Camera = Camera.open();
		try
		{
			m_Camera.setPreviewDisplay(holder);
		}
		catch (IOException e)
		{
			e.printStackTrace();
			m_Camera.release();
			m_Camera = null;
		}
	}

	/*
     * Callback invoked when the Surface has been destroyed and must no longer
     * be touched. WARNING: after this method returns, the Surface/Canvas must
     * never be touched again!
     */
	@Override
	public void surfaceDestroyed(SurfaceHolder holder)
	{
		m_Camera.stopPreview();
		m_Camera.release();
		m_Camera = null;
	}
	
	private Size getOptimalPreviewSize(List<Size> sizes, int w, int h)
	{
        final double ASPECT_TOLERANCE = 0.05;
        double targetRatio = (double) w / h;
        if (sizes == null) return null;

        Size optimalSize = null;
        double minDiff = Double.MAX_VALUE;

        int targetHeight = h;

        // Try to find an size match aspect ratio and size
        for (Size size : sizes)
        {
            double ratio = (double) size.width / size.height;
            if (Math.abs(ratio - targetRatio) > ASPECT_TOLERANCE) continue;
            if (Math.abs(size.height - targetHeight) < minDiff)
            {
                optimalSize = size;
                minDiff = Math.abs(size.height - targetHeight);
            }
        }

        // Cannot find the one match the aspect ratio, ignore the requirement
        if (optimalSize == null)
        {
            minDiff = Double.MAX_VALUE;
            for (Size size : sizes)
            {
                if (Math.abs(size.height - targetHeight) < minDiff)
                {
                    optimalSize = size;
                    minDiff = Math.abs(size.height - targetHeight);
                }
            }
        }
        return optimalSize;
    }
	
	public void TakePicture()
	{
		if (m_Camera != null && m_CanDoPhoto)
		{
			m_CanDoPhoto = false;
			m_Camera.takePicture(shutterCallback, rawCallback, jpegCallback);
		}
	}
	
	ShutterCallback shutterCallback = new ShutterCallback()
	{
		public void onShutter()
		{
		    // TODO Do something when the shutter closes.
		}
	};
		 
	PictureCallback rawCallback = new PictureCallback()
	{
		public void onPictureTaken(byte[] _data, Camera _camera)
		{
		    // TODO Do something with the image RAW data.
		}
	};
		 
	PictureCallback jpegCallback = new PictureCallback()
	{
		public void onPictureTaken(byte[] _data, Camera _camera)
		{
			String path = Environment.getExternalStorageDirectory().toString();
			File file = new File(path, "MyTestPhoto.jpg");
			try {
				FileOutputStream fOut = new FileOutputStream(file);
				fOut.write(_data);
				fOut.flush();
	            fOut.close();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			m_CanDoPhoto = true;
			m_Camera.startPreview();
		}
	};
}
